<?php

namespace Custom\ProductManagement\Console\Command;

use Custom\ProductManagement\Model\Logger;
use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Framework\App\Area;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\State;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Custom ProductManagement no images console command.
 *
 * @category Custom
 * @package  Custom_ProductManagement
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class NoImagesCommand extends Command
{
    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var DirectoryList
     */
    private $directoryList;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var DateTime
     */
    private $dateTime;

    /**
     * @var State
     */
    private $appState;

    /**
     * Object constructor.
     *
     * @param Logger                $logger
     * @param DirectoryList         $directoryList
     * @param CollectionFactory     $collectionFactory
     * @param StoreManagerInterface $storeManager
     * @param DateTime              $dateTime
     * @param State                 $appState
     */
    public function __construct(
        Logger $logger,
        DirectoryList $directoryList,
        CollectionFactory $collectionFactory,
        StoreManagerInterface $storeManager,
        DateTime $dateTime,
        State $appState
    ) {
        $this->logger = $logger;
        $this->directoryList = $directoryList;
        $this->collectionFactory = $collectionFactory;
        $this->storeManager = $storeManager;
        $this->dateTime = $dateTime;
        $this->appState = $appState;

        parent::__construct();
    }

    /**
     * {@inheritdoc}
     *
     * @return void
     * @throws \InvalidArgumentException
     */
    protected function configure()
    {
        $this
            ->setName('custom:productmanagement:noimages')
            ->setDescription('Create CSV file with a list of products with out images');

        parent::configure();
    }

    /**
     * {@inheritdoc}
     *
     * @return int
     * @throws \Exception
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    ) {
        $this->logger->logInfo('Creating report file...');

        try {
            $this->appState->getAreaCode();
        } catch (LocalizedException $e) {
            $this->appState->setAreaCode(Area::AREA_ADMINHTML);
        }

        try {
            $filename = $this->generateReport();

            $this->logger->logInfo('Report file has been created.');
            $this->logger->logInfo($filename);
        } catch (\Exception $e) {
            $this->logger->logError('During report file creation process error has occurred.');
            $this->logger->logError('Details: ' . $e->getMessage());

            return -1;
        }

        return 0;
    }

    private function generateReport()
    {
        $headers = [
            __('Sku'),
            __('Name'),
            __('Barcode'),
            __('Price'),
        ];

        $reportDir = $this->directoryList->getPath(DirectoryList::VAR_DIR)
            . '/report';
        if (!file_exists($reportDir)) {
            mkdir($reportDir, 0777);
        }

        $outputFile = $reportDir . '/'
            . $this->dateTime->date('Ymd_Hi') . '_noimages_report.csv';

        $handle = fopen($outputFile, 'w');
        fputcsv($handle, $headers);

        foreach ($this->collectReportData() as $rowData) {
            fputcsv($handle, $rowData);
        }

        return $outputFile;
    }

    private function collectReportData()
    {
        $collection = $this->collectionFactory->create();
        $collection
            //->setStoreId(Store::DEFAULT_STORE_ID)
	    ->setStoreId(19)
            ->addAttributeToSelect(['sku', 'barcode', 'name', 'price', 'image', 'status'])
            ->addAttributeToFilter('status', ['eq' => Status::STATUS_ENABLED])
            ->addAttributeToFilter(
                [
                    [
                        'attribute' => 'image',
                        'null' => '1',
                    ],
                    [
                        'attribute' => 'image',
                        'nlike' => '%/%/%',
                    ],
                ],
                null,
                'left'
            );

        $data = [];
        foreach ($collection as $product) {
            $data[] = [
                $product->getSku(),
                $product->getName(),
                ' ' . $product->getBarcode(),
                $product->getPrice(),
            ];
        }

        return $data;
    }
}
