<?php

namespace Custom\ProductManagement\Model;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\App\State;
use Exception;
use Magento\Catalog\Model\ProductFactory;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\Catalog\Model\Product\Attribute\Source\Status as ProductStatus;
use Magento\Framework\App\Area;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Custom ProductManagement product status updater.
 *
 * @category Custom
 * @package  Custom_ProductManagement
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class Status
{
    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var State
     */
    private $appState;

    /**
     * @var ProductFactory
     */
    private $productFactory;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var string
     */
    private $attribute;

    /**
     * @var array
     */
    private $attributeValues = [];

    /**
     * @var int
     */
    private $status;

    /**
     * @var int
     */
    private $rowNumber;

    /**
     * @var int
     */
    private $rowCount;

    /**
     * Import constructor.
     *
     * @param Logger                $logger
     * @param State                 $appState
     * @param ProductFactory        $productFactory
     * @param CollectionFactory     $collectionFactory
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        Logger $logger,
        State $appState,
        ProductFactory $productFactory,
        CollectionFactory $collectionFactory,
        StoreManagerInterface $storeManager
    ) {
        $this->logger = $logger;
        $this->appState = $appState;
        $this->productFactory = $productFactory;
        $this->collectionFactory = $collectionFactory;
        $this->storeManager = $storeManager;
    }

    /**
     * @param string $attribute
     *
     * @return Status
     */
    public function setAttribute($attribute)
    {
        $this->attribute = $attribute;

        return $this;
    }

    /**
     * @param array $attributeValues
     *
     * @return Status
     */
    public function setAttributeValues($attributeValues)
    {
        $this->attributeValues = $attributeValues;

        return $this;
    }

    /**
     * @param int $status
     *
     * @return Status
     */
    public function setProductStatus($status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * @return Status
     * @throws Exception
     * @throws LocalizedException
     */
    public function process()
    {
        try {
            $code = $this->appState->getAreaCode();
        } catch (LocalizedException $e) {
            $this->appState->setAreaCode(Area::AREA_ADMINHTML);
        }

        $this->rowCount = count($this->attributeValues);
        for ($i = 0; $i < $this->rowCount; $i++) {
            $this->rowNumber = $i + 1;

            $attributeValue = $this->attributeValues[$i];
            $attributeValue = trim($attributeValue);

            if (empty($attributeValue)) {
                $this->logger->logError(
                    sprintf(
                        '[%s/%s] Skipping, empty value.',
                        $this->rowNumber,
                        $this->rowCount
                    )
                );
                continue;
            }

            $collection = $this->collectionFactory->create();
            $collection->addAttributeToFilter($this->attribute,
                ['eq' => $attributeValue]);

            if ($collection->getSize() === 0) {
                $this->logger->logError(
                    sprintf(
                        '[%s/%s] Error, product matching criteria "%s": "%s" not found.',
                        $this->rowNumber,
                        $this->rowCount,
                        $this->attribute,
                        $attributeValue
                    )
                );
                continue;
            } elseif ($collection->getSize() > 1) {
                $this->logger->logError(
                    sprintf(
                        '[%s/%s] Error, more than 1 product matching criteria "%s": "%s".',
                        $this->rowNumber,
                        $this->rowCount,
                        $this->attribute,
                        $attributeValue
                    )
                );
                continue;
            }

            $matchedProduct = $collection->getFirstItem();

            $this->updateScope(
                $matchedProduct->getId(),
                Store::DEFAULT_STORE_ID,
                'Admin Scope'
            );

            $stores = $this->storeManager->getStores();
            foreach ($stores as $store) {
                $this->updateScope(
                    $matchedProduct->getId(),
                    $store->getId(),
                    $store->getName()
                );
            }
        }

        return $this;
    }

    /**
     * @param int    $productId
     * @param int    $storeId
     * @param string $scopeName
     *
     * @return Status
     * @throws Exception
     */
    private function updateScope($productId, $storeId, $scopeName)
    {
        $product = $this->productFactory->create()
            ->setStoreId($storeId)
            ->load($productId);

        if ((int)$product->getStatus() === (int)$this->status) {
            $this->logger->logInfo(
                sprintf(
                    '[%s/%s] Skipping, product "%s" is already %s on "%s".',
                    $this->rowNumber,
                    $this->rowCount,
                    $product->getSku(),
                    $this->status === ProductStatus::STATUS_ENABLED ? 'activated' : 'deactivated',
                    $scopeName
                )
            );

            return $this;
        }

    	$product
            ->setStatus($this->status)
            ->save();

        $this->logger->logInfo(
            sprintf(
                '[%s/%s] Product "%s" has been %s on "%s".',
                $this->rowNumber,
                $this->rowCount,
                $product->getSku(),
                $this->status === ProductStatus::STATUS_ENABLED ? 'activated' : 'deactivated',
                $scopeName
            )
        );

        return $this;
    }
}

